@@ -550,6 +550,20 @@ Java_com_introlab_rtabmap_RTABMapLib_setAppendMode(
 		UERROR("native_application is null!");
 	}
 }
+JNIEXPORT void JNICALL
+Java_com_introlab_rtabmap_RTABMapLib_setUpstreamRelocalizationAccThr(
+		JNIEnv*, jclass, jlong native_application, float value)
+{
+	if(native_application)
+	{
+		return native(native_application)->setUpstreamRelocalizationAccThr(value);
+	}
+	else
+	{
+		UERROR("native_application is null!");
+	}
+}
+
 JNIEXPORT void JNICALL
 Java_com_introlab_rtabmap_RTABMapLib_setDataRecorderMode(
 		JNIEnv*, jclass, jlong native_application, bool enabled)
@@ -807,6 +821,7 @@ Java_com_introlab_rtabmap_RTABMapLib_exportMesh(
 					optimizedMinClusterSize,
 					optimizedMaxTextureDistance,
 					optimizedMinTextureClusterSize,
+					0,
 					blockRendering);
 	}
 	else
@@ -866,22 +881,6 @@ Java_com_introlab_rtabmap_RTABMapLib_postProcessing(
 	}
 }
 
-JNIEXPORT void JNICALL
-Java_com_introlab_rtabmap_RTABMapLib_postCameraPoseEvent(
-		JNIEnv* env, jclass, jlong native_application,
-		float x, float y, float z, float qx, float qy, float qz, float qw, double stamp)
-{
-	if(native_application)
-	{
-		native(native_application)->postCameraPoseEvent(x,y,z,qx,qy,qz,qw, stamp);
-	}
-	else
-	{
-		UERROR("native_application is null!");
-		return;
-	}
-}
-
 JNIEXPORT void JNICALL
 Java_com_introlab_rtabmap_RTABMapLib_postOdometryEvent(
 		JNIEnv* env, jclass, jlong native_application,
