@@ -25,6 +25,7 @@ ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include <rtabmap/core/SensorEvent.h>
 #include "rtabmap/core/DBReader.h"
 #include "rtabmap/core/DBDriver.h"
 
@@ -34,7 +35,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <rtabmap/utilite/UConversion.h>
 #include <rtabmap/utilite/UEventsManager.h>
 
-#include "rtabmap/core/CameraEvent.h"
 #include "rtabmap/core/RtabmapEvent.h"
 #include "rtabmap/core/OdometryEvent.h"
 #include "rtabmap/core/util3d.h"
@@ -48,14 +48,15 @@ DBReader::DBReader(const std::string & databasePath,
 				   bool ignoreGoalDelay,
 				   bool goalsIgnored,
 				   int startId,
-				   int cameraIndex,
+				   const std::vector<unsigned int> & cameraIndices,
 				   int stopId,
 				   bool intermediateNodesIgnored,
 				   bool landmarksIgnored,
 				   bool featuresIgnored,
 				   int startMapId,
 				   int stopMapId,
-				   bool priorsIgnored) :
+				   bool priorsIgnored,
+				   const std::vector<Transform> & cameraLocalTransformOverrides) :
 	Camera(frameRate),
 	_paths(uSplit(databasePath, ';')),
 	_odometryIgnored(odometryIgnored),
@@ -63,13 +64,14 @@ DBReader::DBReader(const std::string & databasePath,
 	_goalsIgnored(goalsIgnored),
 	_startId(startId),
 	_stopId(stopId),
-	_cameraIndex(cameraIndex),
+	_cameraIndices(cameraIndices),
 	_intermediateNodesIgnored(intermediateNodesIgnored),
 	_landmarksIgnored(landmarksIgnored),
 	_featuresIgnored(featuresIgnored),
 	_priorsIgnored(priorsIgnored),
 	_startMapId(startMapId),
 	_stopMapId(stopMapId),
+	_cameraLocalTransformOverrides(cameraLocalTransformOverrides),
 	_dbDriver(0),
 	_currentId(_ids.end()),
 	_previousMapId(-1),
@@ -77,15 +79,7 @@ DBReader::DBReader(const std::string & databasePath,
 	_previousMapID(0),
 	_calibrated(false)
 {
-	if(_stopId>0 && _stopId<_startId)
-	{
-		_stopId = _startId;
-	}
-
-	if(_stopMapId>-1 && _stopMapId<_startMapId)
-	{
-		_stopMapId = _startMapId;
-	}
+	checkArguments();
 }
 
 DBReader::DBReader(const std::list<std::string> & databasePaths,
@@ -94,14 +88,15 @@ DBReader::DBReader(const std::list<std::string> & databasePaths,
 				   bool ignoreGoalDelay,
 				   bool goalsIgnored,
 				   int startId,
-				   int cameraIndex,
+				   const std::vector<unsigned int> & cameraIndices,
 				   int stopId,
 				   bool intermediateNodesIgnored,
 				   bool landmarksIgnored,
 				   bool featuresIgnored,
 				   int startMapId,
 				   int stopMapId,
-				   bool priorsIgnored) :
+				   bool priorsIgnored,
+				   const std::vector<Transform> & cameraLocalTransformOverrides) :
 	Camera(frameRate),
    _paths(databasePaths),
 	_odometryIgnored(odometryIgnored),
@@ -109,19 +104,25 @@ DBReader::DBReader(const std::list<std::string> & databasePaths,
 	_goalsIgnored(goalsIgnored),
 	_startId(startId),
 	_stopId(stopId),
-	_cameraIndex(cameraIndex),
+	_cameraIndices(cameraIndices),
 	_intermediateNodesIgnored(intermediateNodesIgnored),
 	_landmarksIgnored(landmarksIgnored),
 	_featuresIgnored(featuresIgnored),
 	_priorsIgnored(priorsIgnored),
 	_startMapId(startMapId),
 	_stopMapId(stopMapId),
+	_cameraLocalTransformOverrides(cameraLocalTransformOverrides),
 	_dbDriver(0),
 	_currentId(_ids.end()),
 	_previousMapId(-1),
 	_previousStamp(0),
 	_previousMapID(0),
 	_calibrated(false)
+{
+	checkArguments();
+}
+
+void DBReader::checkArguments()
 {
 	if(_stopId>0 && _stopId<_startId)
 	{
@@ -132,6 +133,28 @@ DBReader::DBReader(const std::list<std::string> & databasePaths,
 	{
 		_stopMapId = _startMapId;
 	}
+
+	if(!_cameraLocalTransformOverrides.empty())
+	{
+		if(!_cameraIndices.empty() &&
+			_cameraIndices.size() != _cameraLocalTransformOverrides.size())
+		{
+			UERROR("Camera local transform overrides (%d) are not the same size than the camera indices (%d). The overrides are ignored.",
+				_cameraLocalTransformOverrides.size(),
+				_cameraIndices.size()
+			);
+			_cameraLocalTransformOverrides.clear();
+		}
+		for(size_t i=0; i<_cameraLocalTransformOverrides.size(); ++i)
+		{
+			if(_cameraLocalTransformOverrides[i].isNull())
+			{
+				UERROR("Camera local transform overrides vector cannot contains null transforms! Clearing overrides.");
+				_cameraLocalTransformOverrides.clear();
+				break;
+			}
+		}
+	}
 }
 
 DBReader::~DBReader()
@@ -144,8 +167,8 @@ DBReader::~DBReader()
 }
 
 bool DBReader::init(
-		const std::string & calibrationFolder,
-		const std::string & cameraName)
+		const std::string &,
+		const std::string &)
 {
 	if(_dbDriver)
 	{
@@ -268,7 +291,13 @@ std::string DBReader::getSerial() const
 	return "DBReader";
 }
 
-SensorData DBReader::captureImage(CameraInfo * info)
+bool DBReader::getPose(double stamp, Transform & pose, cv::Mat & covariance, double maxWaitTime)
+{
+	UERROR("DBReader only provides pose when capturing data, it cannot provide asynchronous pose.");
+	return false;
+}
+
+SensorData DBReader::captureImage(SensorCaptureInfo * info)
 {
 	SensorData data = this->getNextData(info);
 	if(data.id()>0 && _stopId>0 && data.id() > _stopId)
@@ -370,7 +399,7 @@ SensorData DBReader::captureImage(CameraInfo * info)
 	return data;
 }
 
-SensorData DBReader::getNextData(CameraInfo * info)
+SensorData DBReader::getNextData(SensorCaptureInfo * info)
 {
 	SensorData data;
 	if(_dbDriver)
@@ -551,34 +580,134 @@ SensorData DBReader::getNextData(CameraInfo * info)
 			}
 
 			data.uncompressData();
-			if(data.cameraModels().size() > 1 &&
-				_cameraIndex >= 0)
+			std::map<int, int> cameraOldNewIndices;
+			std::vector<CameraModel> dbModels = data.cameraModels();
+			if(dbModels.empty() && !data.stereoCameraModels().empty())
 			{
-				if(_cameraIndex < (int)data.cameraModels().size())
+				for(size_t i=0; i<data.stereoCameraModels().size(); ++i)
 				{
-					// select one camera
-					int subImageWidth = data.imageRaw().cols/data.cameraModels().size();
-					cv::Mat image;
+					dbModels.push_back(data.stereoCameraModels()[i].left());
+				}
+			}
+
+			if(!_cameraLocalTransformOverrides.empty() && 
+				!_cameraIndices.empty() &&
+				_cameraIndices.size() != _cameraLocalTransformOverrides.size())
+			{
+				UERROR("Camera local transform overrides (%d) are not the same size than the camera indices (%d). The overrides are ignored.",
+					_cameraLocalTransformOverrides.size(),
+					_cameraIndices.size()
+				);
+				_cameraLocalTransformOverrides.clear();
+			}
+
+			std::vector<Transform> combinedLocalTransforms;
+			if(dbModels.size() > 1 && !_cameraIndices.empty())
+			{
+				// update images and local transforms
+				cv::Mat combinedImages;
+				cv::Mat combinedDepthImages;
+				std::vector<CameraModel> combinedModels;
+				std::vector<StereoCameraModel> combinedStereoModels;
+				for(size_t i=0; i<_cameraIndices.size(); ++i)
+				{
+					UASSERT_MSG(_cameraIndices[i] < dbModels.size(), uFormat("DBReader: camera index %ld is not valid (should be between 0 and %ld)",
+						_cameraIndices[i], dbModels.size()-1).c_str());
+
+					int addedCameras = std::max(combinedModels.size(), combinedStereoModels.size());
+
+					int subImageWidth = data.imageRaw().cols/dbModels.size();
 					UASSERT(!data.imageRaw().empty() &&
-							data.imageRaw().cols % data.cameraModels().size() == 0 &&
-							_cameraIndex*subImageWidth < data.imageRaw().cols);
-					image= cv::Mat(data.imageRaw(),
-						   cv::Rect(_cameraIndex*subImageWidth, 0, subImageWidth, data.imageRaw().rows)).clone();
+							data.imageRaw().cols % dbModels.size() == 0 &&
+							(int)_cameraIndices[i]*subImageWidth < data.imageRaw().cols);
+					if(combinedImages.empty())
+					{
+						// initialize with first camera
+						combinedImages = cv::Mat(data.imageRaw().rows, subImageWidth*(_cameraIndices.size()-i), data.imageRaw().type());
+					}
+
+					cv::Mat fromROI = cv::Mat(data.imageRaw(), cv::Rect(_cameraIndices[i]*subImageWidth, 0, subImageWidth, data.imageRaw().rows));
+					cv::Mat toROI = cv::Mat(combinedImages, cv::Rect(addedCameras*subImageWidth, 0, subImageWidth, combinedImages.rows));
+					fromROI.copyTo(toROI);
 
 					cv::Mat depth;
 					if(!data.depthOrRightRaw().empty())
 					{
-						UASSERT(data.depthOrRightRaw().cols % data.cameraModels().size() == 0 &&
-								subImageWidth == data.depthOrRightRaw().cols/(int)data.cameraModels().size() &&
-								_cameraIndex*subImageWidth < data.depthOrRightRaw().cols);
-						depth = cv::Mat(data.depthOrRightRaw(),
-							    cv::Rect(_cameraIndex*subImageWidth, 0, subImageWidth, data.depthOrRightRaw().rows)).clone();
+						subImageWidth = data.depthOrRightRaw().cols/dbModels.size();
+						UASSERT(data.depthOrRightRaw().cols % dbModels.size() == 0 &&
+								subImageWidth == data.depthOrRightRaw().cols/(int)dbModels.size() &&
+								(int)_cameraIndices[i]*subImageWidth < data.depthOrRightRaw().cols);
+						if(combinedDepthImages.empty())
+						{
+							// initialize with first camera
+							combinedDepthImages = cv::Mat(data.depthOrRightRaw().rows, subImageWidth*(_cameraIndices.size()-i), data.depthOrRightRaw().type());
+						}
+						fromROI = cv::Mat(data.depthOrRightRaw(), cv::Rect(_cameraIndices[i]*subImageWidth, 0, subImageWidth, data.depthOrRightRaw().rows));
+						toROI = cv::Mat(combinedDepthImages, cv::Rect(addedCameras*subImageWidth, 0, subImageWidth, combinedDepthImages.rows));
+						fromROI.copyTo(toROI);
 					}
-					data.setRGBDImage(image, depth, data.cameraModels().at(_cameraIndex));
+
+					if(!data.cameraModels().empty())
+					{
+						CameraModel model = data.cameraModels()[_cameraIndices[i]];
+						if(!_cameraLocalTransformOverrides.empty())
+						{
+							model.setLocalTransform(_cameraLocalTransformOverrides[i] * CameraModel::opticalRotation());
+						}
+						combinedModels.push_back(model);
+						combinedLocalTransforms.push_back(model.localTransform());
+					}
+					else
+					{
+						StereoCameraModel stereoModel = data.stereoCameraModels()[_cameraIndices[i]];
+						if(!_cameraLocalTransformOverrides.empty())
+						{
+							stereoModel.setLocalTransform(_cameraLocalTransformOverrides[i] * CameraModel::opticalRotation());
+						}
+						combinedStereoModels.push_back(stereoModel);
+						combinedLocalTransforms.push_back(stereoModel.localTransform());
+					}
+					cameraOldNewIndices.insert(std::make_pair(_cameraIndices[i], i));
+				}
+				if(!combinedModels.empty())
+				{
+					data.setRGBDImage(combinedImages, combinedDepthImages, combinedModels);
 				}
 				else
 				{
-					UWARN("DBReader: Camera index %d doesn't exist! Camera models = %d.", _cameraIndex, (int)data.cameraModels().size());
+					data.setStereoImage(combinedImages, combinedDepthImages, combinedStereoModels);
+				}
+			}
+			else if(!_cameraLocalTransformOverrides.empty() &&
+					_cameraLocalTransformOverrides.size() == dbModels.size())
+			{
+				// just update local transforms
+				std::vector<CameraModel> combinedModels;
+				std::vector<StereoCameraModel> combinedStereoModels;
+				for(size_t i=0; i<dbModels.size(); ++i)
+				{
+					if(!data.cameraModels().empty())
+					{
+						CameraModel model = data.cameraModels()[i];
+						model.setLocalTransform(_cameraLocalTransformOverrides[i] * CameraModel::opticalRotation());
+						combinedModels.push_back(model);
+						combinedLocalTransforms.push_back(model.localTransform());
+					}
+					else
+					{
+						StereoCameraModel stereoModel = data.stereoCameraModels()[i];
+						stereoModel.setLocalTransform(_cameraLocalTransformOverrides[i] * CameraModel::opticalRotation());
+						combinedStereoModels.push_back(stereoModel);
+						combinedLocalTransforms.push_back(stereoModel.localTransform());
+					}
+				}
+				if(!combinedModels.empty())
+				{
+					data.setCameraModels(combinedModels);
+				}
+				else
+				{
+					data.setStereoCameraModels(combinedStereoModels);
 				}
 			}
 			data.setId(seq);
@@ -617,14 +746,62 @@ SensorData DBReader::getNextData(CameraInfo * info)
 			   (keypoints3D.empty() || keypoints.size() == keypoints3D.size()) &&
 			   (descriptors.empty() || (int)keypoints.size() == descriptors.rows))
 			{
-				data.setFeatures(keypoints, keypoints3D, descriptors);
+				if(!cameraOldNewIndices.empty())
+				{
+					cv::Mat newDescriptors;
+					std::vector<cv::KeyPoint> newKeypoints;
+					std::vector<cv::Point3f> newKeypoints3D;
+					UASSERT(!dbModels.empty() && dbModels[0].imageWidth()>0);
+					int subImageWidth = dbModels[0].imageWidth();
+					for(size_t i = 0; i<keypoints.size(); ++i)
+					{
+						int cameraIndex = int(keypoints.at(i).pt.x / subImageWidth);
+						UASSERT_MSG(cameraIndex >= 0 && cameraIndex < (int)dbModels.size(),
+								uFormat("cameraIndex=%d, db models=%d, kpt.x=%f, image width=%d",
+										cameraIndex, (int)dbModels.size(), keypoints[i].pt.x, subImageWidth).c_str());
+						if(cameraOldNewIndices.find(cameraIndex) != cameraOldNewIndices.end())
+						{
+							int newCameraIndex = cameraOldNewIndices.at(cameraIndex);
+							newKeypoints.push_back(keypoints[i]);
+							newKeypoints.back().pt.x += (newCameraIndex-cameraIndex)*subImageWidth;
+							if(!keypoints3D.empty())
+							{
+								cv::Point3f pt = util3d::transformPoint(keypoints3D.at(i), dbModels[cameraIndex].localTransform().inverse());
+								pt = util3d::transformPoint(pt, combinedLocalTransforms[cameraIndex]);
+								newKeypoints3D.push_back(pt);
+							}
+							if(!descriptors.empty())
+							{
+								newDescriptors.push_back(descriptors.row(i));
+							}
+						}
+					}
+					data.setFeatures(newKeypoints, newKeypoints3D, newDescriptors);
+				}
+				else if(!combinedLocalTransforms.empty())
+				{
+					// We are overriding the camra local transforms, let's move 3D words accordingly
+					UASSERT(dbModels.size() == combinedLocalTransforms.size());
+					std::vector<cv::Point3f> newKeypoints3D;
+					for(size_t i = 0; i<keypoints3D.size(); ++i)
+					{
+						cv::Point3f pt = util3d::transformPoint(keypoints3D.at(i), dbModels[i].localTransform().inverse());
+						pt = util3d::transformPoint(pt, combinedLocalTransforms[i]);
+						newKeypoints3D.push_back(pt);
+					}
+					data.setFeatures(keypoints, newKeypoints3D, descriptors);
+				}
+				else
+				{
+					data.setFeatures(keypoints, keypoints3D, descriptors);
+				}
 			}
 			else if(!_featuresIgnored && !keypoints.empty() && (!keypoints3D.empty() || !descriptors.empty()))
 			{
 				UERROR("Missing feature data, features won't be published.");
 			}
 
-			if(data.imageCompressed().empty() && s->getWeight()>=0 && keypoints.empty())
+			if(data.imageRaw().empty() && data.imageCompressed().empty() && s->getWeight()>=0 && keypoints.empty())
 			{
 				UWARN("No image loaded from the database for id=%d!", seq);
 			}
