@@ -236,6 +236,12 @@ const std::map<std::string, std::pair<bool, std::string> > & Parameters::getRemo
 	{
 		// removed parameters
 
+		// 0.21.13
+		removedParameters_.insert(std::make_pair("Vis/ForwardEstOnly",    std::make_pair(false, "")));
+
+		// 0.21.7
+		removedParameters_.insert(std::make_pair("SIFT/NFeatures",    std::make_pair(false, "")));
+
 		// 0.21.3
 		removedParameters_.insert(std::make_pair("GridGlobal/FullUpdate",    std::make_pair(false, "")));
 
@@ -682,6 +688,18 @@ ParametersMap Parameters::parseArguments(int argc, char * argv[], bool onlyParam
 				std::cout << str << std::setw(spacing - str.size()) << "true" << std::endl;
 #else
 				std::cout << str << std::setw(spacing - str.size()) << "false" << std::endl;
+#endif
+				str = "With libLAS:";
+#ifdef RTABMAP_LIBLAS
+				std::cout << str << std::setw(spacing - str.size()) << "true" << std::endl;
+#else
+				std::cout << str << std::setw(spacing - str.size()) << "false" << std::endl;
+#endif
+				str = "With CudaSift:";
+#ifdef RTABMAP_CUDASIFT
+				std::cout << str << std::setw(spacing - str.size()) << "true" << std::endl;
+#else
+				std::cout << str << std::setw(spacing - str.size()) << "false" << std::endl;
 #endif
 				str = "With TORO:";
 #ifdef RTABMAP_TORO
@@ -718,6 +736,12 @@ ParametersMap Parameters::parseArguments(int argc, char * argv[], bool onlyParam
 				std::cout << str << std::setw(spacing - str.size()) << "true" << std::endl;
 #else
 				std::cout << str << std::setw(spacing - str.size()) << "false" << std::endl;
+#endif
+				str = "With OpenNI:";
+#ifdef RTABMAP_OPENNI
+				std::cout << str << std::setw(spacing - str.size()) << "true" << std::endl;
+#else
+				std::cout << str << std::setw(spacing - str.size()) << "false" << std::endl;
 #endif
 				str = "With OpenNI2:";
 #ifdef RTABMAP_OPENNI2
@@ -802,6 +826,12 @@ ParametersMap Parameters::parseArguments(int argc, char * argv[], bool onlyParam
 				std::cout << str << std::setw(spacing - str.size()) << "true" << std::endl;
 #else
 				std::cout << str << std::setw(spacing - str.size()) << "false" << std::endl;
+#endif
+				str = "With XVisio SDK:";
+#ifdef RTABMAP_XVSDK
+				std::cout << str << std::setw(spacing - str.size()) << "true" << std::endl;
+#else
+				std::cout << str << std::setw(spacing - str.size()) << "false" << std::endl;
 #endif
 				str = "With libpointmatcher:";
 #ifdef RTABMAP_POINTMATCHER
@@ -1172,11 +1202,8 @@ ParametersMap Parameters::parseArguments(int argc, char * argv[], bool onlyParam
 	return out;
 }
 
-
-void Parameters::readINI(const std::string & configFile, ParametersMap & parameters, bool modifiedOnly)
+void readINIImpl(const CSimpleIniA & ini, const std::string & configFilePath, ParametersMap & parameters, bool modifiedOnly)
 {
-	CSimpleIniA ini;
-	ini.LoadFile(configFile.c_str());
 	const CSimpleIniA::TKeyVal * keyValMap = ini.GetSection("Core");
 	if(keyValMap)
 	{
@@ -1191,12 +1218,12 @@ void Parameters::readINI(const std::string & configFile, ParametersMap & paramet
 				{
 					if(!RTABMAP_VERSION_COMPARE(std::atoi(version[0].c_str()), std::atoi(version[1].c_str()), std::atoi(version[2].c_str())))
 					{
-						if(configFile.find(".rtabmap") != std::string::npos)
+						if(configFilePath.find(".rtabmap") != std::string::npos)
 						{
 							UWARN("Version in the config file \"%s\" is more recent (\"%s\") than "
 								   "current RTAB-Map version used (\"%s\"). The config file will be upgraded "
 								   "to new version.",
-								   configFile.c_str(),
+								   configFilePath.c_str(),
 								   (*iter).second,
 								   RTABMAP_VERSION);
 						}
@@ -1205,7 +1232,7 @@ void Parameters::readINI(const std::string & configFile, ParametersMap & paramet
 							UERROR("Version in the config file \"%s\" is more recent (\"%s\") than "
 								   "current RTAB-Map version used (\"%s\"). New parameters (if there are some) will "
 								   "be ignored.",
-								   configFile.c_str(),
+								   configFilePath.c_str(),
 								   (*iter).second,
 								   RTABMAP_VERSION);
 						}
@@ -1255,11 +1282,26 @@ void Parameters::readINI(const std::string & configFile, ParametersMap & paramet
 	else
 	{
 		ULOGGER_WARN("Section \"Core\" in %s doesn't exist... "
-				    "Ignore this warning if the ini file does not exist yet. "
-				    "The ini file will be automatically created when rtabmap will close.", configFile.c_str());
+					"Ignore this warning if the ini file does not exist yet. "
+					"The ini file will be automatically created when rtabmap will close.", configFilePath.c_str());
 	}
 }
 
+
+void Parameters::readINI(const std::string & configFile, ParametersMap & parameters, bool modifiedOnly)
+{
+	CSimpleIniA ini;
+	ini.LoadFile(configFile.c_str());
+	readINIImpl(ini, configFile, parameters, modifiedOnly);
+}
+
+void Parameters::readINIStr(const std::string & configContent, ParametersMap & parameters, bool modifiedOnly)
+{
+	CSimpleIniA ini;
+	ini.LoadData(configContent);
+	readINIImpl(ini, "", parameters, modifiedOnly);
+}
+
 void Parameters::writeINI(const std::string & configFile, const ParametersMap & parameters)
 {
 	CSimpleIniA ini;
