@@ -28,7 +28,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <rtabmap/core/camera/CameraStereoZed.h>
 #include <rtabmap/utilite/UTimer.h>
 #include <rtabmap/utilite/UThread.h>
-#include <rtabmap/utilite/UEventsManager.h>
 #include <rtabmap/utilite/UConversion.h>
 
 #ifdef RTABMAP_ZED
@@ -167,12 +166,13 @@ IMU zedIMUtoIMU(const sl::SensorsData & sensorData, const Transform & imuLocalTr
 class ZedIMUThread: public UThread
 {
 public:
-	ZedIMUThread(float rate, sl::Camera * zed, const Transform & imuLocalTransform, bool accurate)
+	ZedIMUThread(float rate, sl::Camera * zed, CameraStereoZed * camera, const Transform & imuLocalTransform, bool accurate)
 	{
 		UASSERT(rate > 0.0f);
-		UASSERT(zed != 0);
+		UASSERT(zed != 0 && camera != 0);
 		rate_ = rate;
 		zed_= zed;
+		camera_ = camera;
 		accurate_ = accurate;
 		imuLocalTransform_ = imuLocalTransform;
 	}
@@ -212,19 +212,20 @@ private:
 		bool res = zed_->getIMUData(imudata, sl::TIME_REFERENCE_IMAGE);
 		if(res == sl::SUCCESS && imudata.valid)
 		{
-			UEventsManager::post(new IMUEvent(zedIMUtoIMU(imudata, imuLocalTransform_), UTimer::now()));
+			this->postInterIMU(zedIMUtoIMU(imudata, imuLocalTransform_), UTimer::now());
 		}
 #else
         sl::SensorsData sensordata;
-        sl::ERROR_CODE res = zed_->getSensorsData(sensordata, sl::TIME_REFERENCE::IMAGE);
+        sl::ERROR_CODE res = zed_->getSensorsData(sensordata, sl::TIME_REFERENCE::CURRENT);
         if(res == sl::ERROR_CODE::SUCCESS && sensordata.imu.is_available)
         {
-            UEventsManager::post(new IMUEvent(zedIMUtoIMU(sensordata, imuLocalTransform_), UTimer::now()));
+        	camera_->postInterIMUPublic(zedIMUtoIMU(sensordata, imuLocalTransform_), double(sensordata.imu.timestamp.getNanoseconds())/10e8);
         }
 #endif
 	}
 	float rate_;
 	sl::Camera * zed_;
+	CameraStereoZed * camera_;
 	bool accurate_;
 	Transform imuLocalTransform_;
 	UTimer frameRateTimer_;
@@ -279,22 +280,35 @@ CameraStereoZed::CameraStereoZed(
 	computeOdometry_(computeOdometry),
 	lost_(true),
 	force3DoF_(odomForce3DoF),
-	publishInterIMU_(false),
 	imuPublishingThread_(0)
 #endif
 {
 	UDEBUG("");
 #ifdef RTABMAP_ZED
 #if ZED_SDK_MAJOR_VERSION < 4
-	if(resolution_ == 3)
+	if(resolution_ == 1 || resolution_ == 2) // HD2K, HD1080
 	{
-		resolution_ = 2;
+		resolution_ -= 1; // HD2K=0, HD1080=1
 	}
-	else if(resolution_ == 5)
+	if(resolution_ == 3) // HD1200
 	{
-		resolution_ = 3;
+		resolution_ = 1; // HD1080=1
+	}
+	if(resolution_ == 4 || resolution_ == -1)
+	{
+		resolution_ = 2; // HD720=2
+	}
+	else if(resolution_ == 5 || resolution_ == 6) // SVGA, VGA
+	{
+		resolution_ = 3; // VGA=3
+	}
+#else // ZED=4
+	if(resolution_ == -1)
+	{
+		resolution_ = int(sl::RESOLUTION::AUTO); // AUTO
 	}
 #endif
+
 #if ZED_SDK_MAJOR_VERSION < 3
 	UASSERT(resolution_ >= sl::RESOLUTION_HD2K && resolution_ <sl::RESOLUTION_LAST);
 	UASSERT(quality_ >= sl::DEPTH_MODE_NONE && quality_ <sl::DEPTH_MODE_LAST);
@@ -304,12 +318,13 @@ CameraStereoZed::CameraStereoZed(
     sl::RESOLUTION res = static_cast<sl::RESOLUTION>(resolution_);
     sl::DEPTH_MODE qual = static_cast<sl::DEPTH_MODE>(quality_);
 
-    UASSERT(res >= sl::RESOLUTION::HD2K && res < sl::RESOLUTION::LAST);
     UASSERT(qual >= sl::DEPTH_MODE::NONE && qual < sl::DEPTH_MODE::LAST);
 #if ZED_SDK_MAJOR_VERSION < 4
+	UASSERT(res >= sl::RESOLUTION::HD2K && res < sl::RESOLUTION::LAST);
     sl::SENSING_MODE sens = static_cast<sl::SENSING_MODE>(sensingMode_);
     UASSERT(sens >= sl::SENSING_MODE::STANDARD && sens < sl::SENSING_MODE::LAST);
 #else
+	UASSERT(res >= sl::RESOLUTION::HD4K && res < sl::RESOLUTION::LAST);
     UASSERT(sensingMode_ >= 0 && sensingMode_ < 2);
 #endif
     UASSERT(confidenceThr_ >= 0 && confidenceThr_ <=100);
@@ -336,7 +351,13 @@ CameraStereoZed::CameraStereoZed(
 	src_(CameraVideo::kVideoFile),
 	usbDevice_(0),
 	svoFilePath_(filePath),
-	resolution_(2),
+#if ZED_SDK_MAJOR_VERSION < 3
+	resolution_(sl::RESOLUTION_HD720),
+#elif ZED_SDK_MAJOR_VERSION < 4
+	resolution_(sl::RESOLUTION::HD720),
+#else
+	resolution_(int(sl::RESOLUTION::AUTO)),
+#endif
 	quality_(quality),
 	selfCalibration_(selfCalibration),
 	sensingMode_(sensingMode),
@@ -345,7 +366,6 @@ CameraStereoZed::CameraStereoZed(
 	computeOdometry_(computeOdometry),
 	lost_(true),
 	force3DoF_(odomForce3DoF),
-	publishInterIMU_(false),
 	imuPublishingThread_(0)
 #endif
 {
@@ -386,13 +406,6 @@ CameraStereoZed::~CameraStereoZed()
 #endif
 }
 
-void CameraStereoZed::publishInterIMU(bool enabled)
-{
-#ifdef RTABMAP_ZED
-	publishInterIMU_ = enabled;
-#endif
-}
-
 bool CameraStereoZed::init(const std::string & calibrationFolder, const std::string & cameraName)
 {
 	UDEBUG("");
@@ -564,9 +577,9 @@ bool CameraStereoZed::init(const std::string & calibrationFolder, const std::str
 			imuLocalTransform_.prettyPrint().c_str(),
 			zedPoseToTransform(infos.sensors_configuration.camera_imu_transform).prettyPrint().c_str());
 #endif
-		if(publishInterIMU_)
+		if(isInterIMUPublishing())
 		{
-			imuPublishingThread_ = new ZedIMUThread(200, zed_, imuLocalTransform_, true);
+			imuPublishingThread_ = new ZedIMUThread(200, zed_, this, imuLocalTransform_, true);
 			imuPublishingThread_->start();
 		}
 	}
@@ -607,7 +620,7 @@ bool CameraStereoZed::odomProvided() const
 #endif
 }
 
-bool CameraStereoZed::getPose(double stamp, Transform & pose, cv::Mat & covariance)
+bool CameraStereoZed::getPose(double stamp, Transform & pose, cv::Mat & covariance, double maxWaitTime)
 {
 #ifdef RTABMAP_ZED
 
@@ -683,7 +696,7 @@ bool CameraStereoZed::getPose(double stamp, Transform & pose, cv::Mat & covarian
 	return false;
 }
 
-SensorData CameraStereoZed::captureImage(CameraInfo * info)
+SensorData CameraStereoZed::captureImage(SensorCaptureInfo * info)
 {
 	SensorData data;
 #ifdef RTABMAP_ZED
@@ -711,17 +724,19 @@ SensorData CameraStereoZed::captureImage(CameraInfo * info)
 #else
 
 		sl::ERROR_CODE res;
+        sl::Timestamp timestamp;
 		bool imuReceived = true;
 		do
 		{
 			res = zed_->grab(rparam);
+			timestamp = zed_->getTimestamp(sl::TIME_REFERENCE::IMAGE);
 
 			// If the sensor supports IMU, wait IMU to be available before sending data.
 			if(imuPublishingThread_ == 0 && !imuLocalTransform_.isNull())
 			{
 				 sl::SensorsData imudatatmp;
 				res = zed_->getSensorsData(imudatatmp, sl::TIME_REFERENCE::IMAGE);
-				imuReceived = res == sl::ERROR_CODE::SUCCESS && imudatatmp.imu.is_available && imudatatmp.imu.timestamp.data_ns != 0;
+				imuReceived = res == sl::ERROR_CODE::SUCCESS && imudatatmp.imu.is_available && imudatatmp.imu.timestamp.getNanoseconds() != 0;
 			}
 		}
 		while(src_ == CameraVideo::kUsbDevice && (res!=sl::ERROR_CODE::SUCCESS || !imuReceived) && timer.elapsed() < 2.0);
@@ -752,8 +767,11 @@ SensorData CameraStereoZed::captureImage(CameraInfo * info)
                 zed_->retrieveMeasure(tmp,sl::MEASURE::DEPTH);
 #endif
 				slMat2cvMat(tmp).copyTo(depth);
-
+#if ZED_SDK_MAJOR_VERSION < 3
 				data = SensorData(left, depth, stereoModel_.left(), this->getNextSeqID(), UTimer::now());
+#else
+				data = SensorData(left, depth, stereoModel_.left(), this->getNextSeqID(), double(timestamp.getNanoseconds())/10e8);
+#endif
 			}
 			else
 			{
@@ -765,9 +783,15 @@ SensorData CameraStereoZed::captureImage(CameraInfo * info)
 #endif
 				cv::Mat rgbaRight = slMat2cvMat(tmp);
 				cv::Mat right;
-				cv::cvtColor(rgbaRight, right, cv::COLOR_BGRA2GRAY);
-			
+				if(rightGrayScale_)
+					cv::cvtColor(rgbaRight, right, cv::COLOR_BGRA2GRAY);
+				else
+					cv::cvtColor(rgbaRight, right, cv::COLOR_BGRA2BGR);
+#if ZED_SDK_MAJOR_VERSION < 3
 				data = SensorData(left, right, stereoModel_, this->getNextSeqID(), UTimer::now());
+#else
+				data = SensorData(left, right, stereoModel_, this->getNextSeqID(), double(timestamp.getNanoseconds())/10e8);
+#endif
 			}
 
 			if(imuPublishingThread_ == 0)
@@ -803,6 +827,13 @@ SensorData CameraStereoZed::captureImage(CameraInfo * info)
 					info->odomPose = zedPoseToTransform(pose);
 					if (!info->odomPose.isNull())
 					{
+#if ZED_SDK_MAJOR_VERSION >=3
+						if(pose.timestamp != timestamp)
+						{
+							UWARN("Pose retrieve doesn't have same stamp (%ld) than grabbed image (%ld)", pose.timestamp, timestamp);
+						}
+#endif
+
 						//transform from:
 						// x->right, y->down, z->forward
 						//to:
@@ -858,4 +889,16 @@ SensorData CameraStereoZed::captureImage(CameraInfo * info)
 	return data;
 }
 
+void CameraStereoZed::postInterIMUPublic(const IMU & imu, double stamp)
+{
+	postInterIMU(imu, stamp);
+}
+
+void CameraStereoZed::setRightGrayScale(bool enabled)
+{
+#ifdef RTABMAP_ZED
+	rightGrayScale_ = enabled;
+#endif
+}
+
 } // namespace rtabmap
