@@ -710,7 +710,7 @@ std::string CameraStereoZedOC::getSerial() const
 	return "";
 }
 
-SensorData CameraStereoZedOC::captureImage(CameraInfo * info)
+SensorData CameraStereoZedOC::captureImage(SensorCaptureInfo * info)
 {
 	SensorData data;
 #ifdef RTABMAP_ZEDOC
@@ -755,7 +755,10 @@ SensorData CameraStereoZedOC::captureImage(CameraInfo * info)
 
 					// ----> Extract left and right images from side-by-side
 					left = frameBGR(cv::Rect(0, 0, frameBGR.cols / 2, frameBGR.rows));
-					cv::cvtColor(frameBGR(cv::Rect(frameBGR.cols / 2, 0, frameBGR.cols / 2, frameBGR.rows)),right,cv::COLOR_BGR2GRAY);
+					if(rightGrayScale_)
+						cv::cvtColor(frameBGR(cv::Rect(frameBGR.cols / 2, 0, frameBGR.cols / 2, frameBGR.rows)),right,cv::COLOR_BGR2GRAY);
+					else
+						right = frameBGR(cv::Rect(frameBGR.cols / 2, 0, frameBGR.cols / 2, frameBGR.rows));
 					// <---- Extract left and right images from side-by-side
 
 					if(stereoModel_.isValidForRectification())
@@ -792,4 +795,11 @@ SensorData CameraStereoZedOC::captureImage(CameraInfo * info)
 	return data;
 }
 
+void CameraStereoZedOC::setRightGrayScale(bool enabled)
+{
+#ifdef RTABMAP_ZEDOC
+	rightGrayScale_ = enabled;
+#endif
+}
+
 } // namespace rtabmap
