@@ -2202,7 +2202,79 @@ void NMS(
     }
 }
 
-void rotateImagesUpsideUpIfNecessary(
+std::vector<int> SSC(
+	const std::vector<cv::KeyPoint> & keypoints, int maxKeypoints, float tolerance, int cols, int rows, const std::vector<int> & indx)
+{
+	bool useIndx = keypoints.size() == indx.size();
+
+	// several temp expression variables to simplify solution equation
+	int exp1 = rows + cols + 2*maxKeypoints;
+	long long exp2 = ((long long)4*cols + (long long)4*maxKeypoints + (long long)4*rows*maxKeypoints + (long long)rows*rows + (long long)cols*cols - (long long)2*rows*cols + (long long)4*rows*cols*maxKeypoints);
+	double exp3 = sqrt(exp2);
+	double exp4 = maxKeypoints - 1;
+
+	double sol1 = -round((exp1 + exp3) / exp4); // first solution
+	double sol2 = -round((exp1 - exp3) / exp4); // second solution
+
+	// binary search range initialization with positive solution
+	int high = (sol1 > sol2) ? sol1 : sol2;
+	int low = floor(sqrt((double)keypoints.size() / maxKeypoints));
+	low = std::max(1, low);
+
+	int width;
+	int prevWidth = -1;
+
+	unsigned int Kmin = round(maxKeypoints - (maxKeypoints * tolerance));
+	unsigned int Kmax = round(maxKeypoints + (maxKeypoints * tolerance));
+
+	std::vector<int> ResultVec, result;
+	result.reserve(keypoints.size());
+
+	bool complete = false;
+	while(!complete)
+	{
+		width = low + (high - low) / 2;
+		if(width==prevWidth || low>high) // needed to reassure the same radius is not repeated again
+		{
+			ResultVec = result; // return the keypoints from the previous iteration
+			break;
+		}
+		result.clear();
+		double c = (double)width / 2.0; // initializing Grid
+		int numCellCols = floor(cols / c);
+		int numCellRows = floor(rows / c);
+		cv::Mat coveredMask = cv::Mat::zeros(numCellRows + 1, numCellCols + 1, CV_8UC1);
+
+		for(unsigned int i=0; i<keypoints.size(); ++i)
+		{
+			int row = floor(keypoints[useIndx?indx[i]:i].pt.y / c); // get position of the cell current point is located at
+			int col = floor(keypoints[useIndx?indx[i]:i].pt.x / c);
+			if(!coveredMask.at<uchar>(row, col)) // if the cell is not covered
+			{
+				result.push_back(useIndx?indx[i]:i);
+				int rowMin = ((row - floor(width / c)) >= 0) ? (row - floor(width / c)) : 0; // get range which current radius is covering
+				int rowMax = ((row + floor(width / c)) <= numCellRows) ? (row + floor(width / c)) : numCellRows;
+				int colMin = ((col - floor(width / c)) >= 0) ? (col - floor(width / c)) : 0;
+				int colMax = ((col + floor(width / c)) <= numCellCols) ? (col + floor(width / c)) : numCellCols;
+				coveredMask(cv::Range(rowMin, rowMax + 1), cv::Range(colMin, colMax + 1)) = 255; // cover cells within the square bounding box with width
+			}
+		}
+
+		if(result.size() >= Kmin && result.size() <= Kmax) // solution found
+		{
+			ResultVec = result;
+			complete = true;
+		}
+		else if(result.size() < Kmin)
+			high = width - 1; // update binary search range
+		else
+			low = width + 1;
+		prevWidth = width;
+	}
+	return ResultVec;
+}
+
+bool rotateImagesUpsideUpIfNecessary(
 	CameraModel & model,
 	cv::Mat & rgb,
 	cv::Mat & depth)
@@ -2216,7 +2288,7 @@ void rotateImagesUpsideUpIfNecessary(
 	{
 		// Return original because of ambiguity for what would be considered up...
 		UDEBUG("Ignoring image rotation as pitch(%f)>Pi/4", pitch);
-		return;
+		return false;
 	}
 	if(roll<0)
 	{
@@ -2291,7 +2363,9 @@ void rotateImagesUpsideUpIfNecessary(
 	else
 	{
 		UDEBUG("ROTATION_0 (roll=%f)", roll);
+		return false;
 	}
+	return true;
 }
 
 }
