@@ -60,6 +60,8 @@ namespace rtabmap {
 class Signature;
 class LoopClosureViewer;
 class Camera;
+class SensorCapture;
+class Lidar;
 class CalibrationDialog;
 class CreateSimpleCalibrationDialog;
 
@@ -74,10 +76,11 @@ public:
 		kPanelCloudRendering = 2,
 		kPanelLogging = 4,
 		kPanelSource = 8,
-		kPanelAll = 15
+		kPanelCalibration = 16,
+		kPanelAll = 31
 	};
 	// TODO, tried to change the name of PANEL_FLAGS to PanelFlags... but signals/slots errors appeared...
-	Q_DECLARE_FLAGS(PANEL_FLAGS, PanelFlag)
+	Q_DECLARE_FLAGS(PANEL_FLAGS, PanelFlag);
 
 	enum Src {
 		kSrcUndef = -1,
@@ -94,6 +97,7 @@ public:
 		kSrcK4W2           = 8,
 		kSrcRealSense2     = 9,
 		kSrcK4A            = 10,
+		kSrcSeerSense      = 11,
 
 		kSrcStereo         = 100,
 		kSrcDC1394         = 100,
@@ -113,7 +117,10 @@ public:
 		kSrcImages         = 201,
 		kSrcVideo          = 202,
 
-		kSrcDatabase       = 300
+		kSrcDatabase       = 300,
+
+		kSrcLidar           = 400,
+		kSrcLidarVLP16      = 400,
 	};
 
 public:
@@ -226,6 +233,7 @@ public:
 	double getSubtractFilteringRadius() const;
 	double getSubtractFilteringAngle() const;
 
+	double getGridUIResolution() const;
 	bool getGridMapShown() const;
 	int getElevationMapShown() const;
 	int getGridMapSensor() const;
@@ -252,6 +260,7 @@ public:
 	PreferencesDialog::Src getSourceDriver() const;
 	QString getSourceDriverStr() const;
 	QString getSourceDevice() const;
+	PreferencesDialog::Src getLidarSourceDriver() const;
 	PreferencesDialog::Src getOdomSourceDriver() const;
 
 	bool isSourceDatabaseStampsUsed() const;
@@ -269,7 +278,9 @@ public:
 	bool isSourceFeatureDetection() const;
 	bool isSourceStereoDepthGenerated() const;
 	bool isSourceStereoExposureCompensation() const;
+	bool isRightGrayScale() const;
 	bool isSourceScanFromDepth() const;
+	bool isSourceScanDeskewing() const;
 	int getSourceScanDownsampleStep() const;
 	double getSourceScanRangeMin() const;
 	double getSourceScanRangeMax() const;
@@ -283,7 +294,8 @@ public:
 	QString getIMUPath() const;
 	int getIMURate() const;
 	Camera * createCamera(bool useRawImages = false, bool useColor = true); // return camera should be deleted if not null
-	Camera * createOdomSensor(Transform & extrinsics, double & timeOffset, float & scaleFactor); // return camera should be deleted if not null
+	SensorCapture * createOdomSensor(Transform & extrinsics, double & timeOffset, float & scaleFactor, double & waitTime); // return odom sensor should be deleted if not null
+	Lidar * createLidar(); // return lidar should be deleted if not null
 
 	int getIgnoredDCComponents() const;
 
@@ -323,6 +335,7 @@ public Q_SLOTS:
 	void calibrate();
 	void calibrateSimple();
 	void calibrateOdomSensorExtrinsics();
+	void testLidar();
 
 private Q_SLOTS:
 	void closeDialog ( QAbstractButton * button );
@@ -331,10 +344,12 @@ private Q_SLOTS:
 	void loadConfigFrom();
 	bool saveConfigTo();
 	void resetConfig();
+	void loadPreset();
 	void makeObsoleteGeneralPanel();
 	void makeObsoleteCloudRenderingPanel();
 	void makeObsoleteLoggingPanel();
 	void makeObsoleteSourcePanel();
+	void makeObsoleteCalibrationPanel();
 	void clicked(const QModelIndex & current, const QModelIndex & previous);
 	void addParameter(int value);
 	void addParameter(bool value);
@@ -344,6 +359,7 @@ private Q_SLOTS:
 	void updateKpROI();
 	void updateStereoDisparityVisibility();
 	void updateFeatureMatchingVisibility();
+	void updateGlobalDescriptorVisibility();
 	void updateOdometryStackedIndex(int index);
 	void useOdomFeatures();
 	void changeWorkingDirectory();
@@ -357,6 +373,7 @@ private Q_SLOTS:
 	void changeSuperPointModelPath();
 	void changePyMatcherPath();
 	void changePyMatcherModel();
+	void changePyDescriptorPath();
 	void changePyDetectorPath();
 	void readSettingsEnd();
 	void setupTreeView();
@@ -386,6 +403,7 @@ private Q_SLOTS:
 	void selectSourceSvoPath();
 	void selectSourceRealsense2JsonPath();
 	void selectSourceDepthaiBlobPath();
+	void selectVlp16PcapPath();
 	void updateSourceGrpVisibility();
 	void testOdometry();
 	void testCamera();
@@ -414,6 +432,7 @@ private:
 	void setupKpRoiPanel();
 	bool parseModel(QList<QGroupBox*> & boxes, QStandardItem * parentItem, int currentLevel, int & absoluteIndex);
 	void resetSettings(QGroupBox * groupBox);
+	void loadPreset(const std::string & presetHexHeader);
 	void addParameter(const QObject * object, int value);
 	void addParameter(const QObject * object, bool value);
 	void addParameter(const QObject * object, double value);
