@@ -45,6 +45,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endif
 #include <QInputDialog>
 #include <QMessageBox>
+#include <QToolTip>
 
 #include <QtCore/QDir>
 #include <QtCore/QDateTime>
@@ -91,7 +92,7 @@ public:
 	}
 	virtual ~NodeItem() {}
 
-	void setColor(const QColor & color)
+	void setColor(const QColor & color, const QString & valueName = QString(), float value = 0.0f)
 	{
 		QPen p = this->pen();
 		p.setColor(color);
@@ -103,6 +104,9 @@ public:
 		QPen pen = _line->pen();
 		pen.setColor(QColor(255-color.red(), 255-color.green(), 255-color.blue()));
 		_line->setPen(pen);
+
+		_valueName = valueName;
+		_value = value;
 	}
 
 	void setRadius(float radius)
@@ -143,14 +147,22 @@ public:
 protected:
 	virtual void hoverEnterEvent ( QGraphicsSceneHoverEvent * event )
 	{
+		QString msg;
 		if(_weight>=0)
 		{
-			this->setToolTip(QString("%1 [map=%2, w=%3] %4").arg(_id).arg(_mapId).arg(_weight).arg(_pose.prettyPrint().c_str()));
+			msg = QString("%1 [map=%2, w=%3]\n%4").arg(_id).arg(_mapId).arg(_weight).arg(_pose.prettyPrint().c_str());
 		}
 		else
 		{
-			this->setToolTip(QString("%1 [map=%2] %3").arg(_id).arg(_mapId).arg(_pose.prettyPrint().c_str()));
+			msg = QString("%1 [map=%2]\n%3").arg(_id).arg(_mapId).arg(_pose.prettyPrint().c_str());
 		}
+		if(!_valueName.isEmpty())
+		{
+			msg += QString("\n%1=%2").arg(_valueName).arg(_value);
+		}
+
+		this->setToolTip(msg);
+		
 		this->setScale(2);
 		QGraphicsEllipseItem::hoverEnterEvent(event);
 	}
@@ -167,6 +179,8 @@ private:
 	int _weight;
 	Transform _pose;
 	QGraphicsLineItem * _line;
+	QString _valueName;
+	float _value;
 };
 
 class NodeGPSItem: public NodeItem
@@ -316,6 +330,7 @@ GraphViewer::GraphViewer(QWidget * parent) :
 		_loopClosureOutlierThr(0),
 		_maxLinkLength(0.02f),
 		_orientationENU(false),
+		_mouseTracking(false),
 		_viewPlane(XY),
 		_ensureFrameVisible(true)
 {
@@ -461,6 +476,10 @@ GraphViewer::GraphViewer(QWidget * parent) :
 	_odomCacheOverlay->setBrush(QBrush(QColor(255, 255, 255, 150)));
 	_odomCacheOverlay->setPen(QPen(Qt::NoPen));
 
+	// Match by default scan colors from DatabaseViewer
+	_highlightedNodes.push_back(QPair<QColor, NodeItem*>(Qt::yellow, nullptr));
+	_highlightedNodes.push_back(QPair<QColor, NodeItem*>(Qt::magenta, nullptr));
+
 	this->restoreDefaults();
 
 	this->fitInView(this->sceneRect(), Qt::KeepAspectRatio);
@@ -700,6 +719,14 @@ void GraphViewer::updateGraph(const std::map<int, Transform> & poses,
 	{
 		if(!iter.value()->isVisible())
 		{
+			for(int i=0; i<_highlightedNodes.size(); ++i)
+			{
+				if(_highlightedNodes[i].second && _highlightedNodes[i].second == iter.value())
+				{
+					_highlightedNodes[i].second = nullptr;
+				}
+			}
+
 			delete iter.value();
 			iter = _nodeItems.erase(iter);
 		}
@@ -1057,11 +1084,16 @@ void GraphViewer::updateMap(const cv::Mat & map8U, float resolution, float xMin,
 }
 
 void GraphViewer::updatePosterior(const std::map<int, float> & posterior, float max, int zValueOffset)
+{
+	updateNodeColorByValue("Posterior Prob", posterior, max, false, zValueOffset);
+}
+
+void GraphViewer::updateNodeColorByValue(const std::string & valueName, const std::map<int, float> & values, float max, bool invertedColorScale, int zValueOffset)
 {
 	//find max
 	if(max <= 0.0f)
 	{
-		for(std::map<int, float>::const_iterator iter = posterior.begin(); iter!=posterior.end(); ++iter)
+		for(std::map<int, float>::const_iterator iter = values.begin(); iter!=values.end(); ++iter)
 		{
 			if(iter->first > 0 && iter->second>max)
 			{
@@ -1073,11 +1105,11 @@ void GraphViewer::updatePosterior(const std::map<int, float> & posterior, float
 	{
 		for(QMap<int, NodeItem*>::iterator iter = _nodeItems.begin(); iter!=_nodeItems.end(); ++iter)
 		{
-			std::map<int,float>::const_iterator jter = posterior.find(iter.key());
-			if(jter != posterior.end())
+			std::map<int,float>::const_iterator jter = values.find(iter.key());
+			if(jter != values.end())
 			{
 				float v = jter->second>max?max:jter->second;
-				iter.value()->setColor(QColor::fromHsvF((1-v/max)*240.0f/360.0f, 1, 1, 1)); //0=red 240=blue
+				iter.value()->setColor(QColor::fromHsvF(( invertedColorScale ? v/max : 1-v/max )*240.0f/360.0f, 1, 1, 1), valueName.c_str(), jter->second); //0=red 240=blue
 				iter.value()->setZValue(iter.value()->zValue()+zValueOffset);
 			}
 		}
@@ -1206,6 +1238,33 @@ void GraphViewer::updateLocalPath(const std::vector<int> & localPath)
 	_localPathRoot->setVisible(wasVisible);
 }
 
+
+void GraphViewer::highlightNode(int nodeId, int highlightIndex)
+{
+	if(highlightIndex<0 || highlightIndex>_highlightedNodes.size())
+	{
+		UERROR("Unsupported highlight color index %d", highlightIndex);
+		return;
+	}
+	for(int i=0; i<_highlightedNodes.size(); ++i)
+	{
+		if(_highlightedNodes[i].second &&
+		   (_highlightedNodes[i].second->id() == nodeId || i == highlightIndex))
+		{
+			// reset to normal color
+			_highlightedNodes[i].second->setColor(_nodeColor);
+			_highlightedNodes[i].second = nullptr;
+		}
+	}
+
+	QMap<int, NodeItem*>::iterator iter = _nodeItems.find(nodeId);
+	if(iter != _nodeItems.end())
+	{
+		iter.value()->setColor(_highlightedNodes[highlightIndex].first);
+		_highlightedNodes[highlightIndex].second = iter.value();
+	}
+}
+
 void GraphViewer::clearGraph()
 {
 	qDeleteAll(_nodeItems);
@@ -1225,6 +1284,11 @@ void GraphViewer::clearGraph()
 	qDeleteAll(_gpsLinkItems);
 	_gpsLinkItems.clear();
 
+	for(int i=0; i<_highlightedNodes.size(); ++i)
+	{
+		_highlightedNodes[i].second=nullptr;
+	}
+
 	_root->resetTransform();
 	_worldMapRotation = 0.0f;
 	_referential->resetTransform();
@@ -1240,6 +1304,11 @@ void GraphViewer::clearMap()
 }
 
 void GraphViewer::clearPosterior()
+{
+	clearNodeColorByValue();
+}
+
+void GraphViewer::clearNodeColorByValue()
 {
 	for(QMap<int, NodeItem*>::iterator iter = _nodeItems.begin(); iter!=_nodeItems.end(); ++iter)
 	{
@@ -1264,6 +1333,10 @@ void GraphViewer::saveSettings(QSettings & settings, const QString & group) cons
 	settings.setValue("node_color", this->getNodeColor());
 	settings.setValue("node_odom_cache_color", this->getNodeOdomCacheColor());
 	settings.setValue("current_goal_color", this->getCurrentGoalColor());
+	for(int i=0; i<_highlightedNodes.size(); ++i)
+	{
+		settings.setValue(QString("highlighting_color_%1").arg(i), _highlightedNodes[i].first);
+	}
 	settings.setValue("neighbor_color", this->getNeighborColor());
 	settings.setValue("global_color", this->getGlobalLoopClosureColor());
 	settings.setValue("local_color", this->getLocalLoopClosureColor());
@@ -1310,6 +1383,10 @@ void GraphViewer::loadSettings(QSettings & settings, const QString & group)
 	this->setNodeColor(settings.value("node_color", this->getNodeColor()).value<QColor>());
 	this->setNodeOdomCacheColor(settings.value("node_odom_cache_color", this->getNodeOdomCacheColor()).value<QColor>());
 	this->setCurrentGoalColor(settings.value("current_goal_color", this->getCurrentGoalColor()).value<QColor>());
+	for(int i=0; i<_highlightedNodes.size(); ++i)
+	{
+		this->setHighlightColor(settings.value(QString("highlighting_color_%1").arg(i), _highlightedNodes[i].first).value<QColor>(), i);
+	}
 	this->setNeighborColor(settings.value("neighbor_color", this->getNeighborColor()).value<QColor>());
 	this->setGlobalLoopClosureColor(settings.value("global_color", this->getGlobalLoopClosureColor()).value<QColor>());
 	this->setLocalLoopClosureColor(settings.value("local_color", this->getLocalLoopClosureColor()).value<QColor>());
@@ -1594,6 +1671,20 @@ void GraphViewer::setGPSColor(const QColor & color)
 		iter.value()->setColor(_gpsPathColor);
 	}
 }
+void GraphViewer::setHighlightColor(const QColor & color, int index)
+{
+	if(index<0 || index > _highlightedNodes.size())
+	{
+		UERROR("Unsupported highlight color index %d", index);
+		return;
+	}
+	_highlightedNodes[index].first = color;
+	NodeItem * node = _highlightedNodes[index].second;
+	if(node != nullptr)
+	{
+		node->setColor(color);
+	}
+}
 void GraphViewer::setIntraSessionLoopColor(const QColor & color)
 {
 	_loopIntraSessionColor = color;
@@ -1763,13 +1854,25 @@ void GraphViewer::restoreDefaults()
 	setNodeRadius(0.01f);
 	setLinkWidth(0.0f);
 	setNodeColor(Qt::blue);
+	setNodeOdomCacheColor(Qt::darkGreen);
+	setCurrentGoalColor(Qt::darkMagenta);
+	setHighlightColor(Qt::yellow, 0);
+	setHighlightColor(Qt::magenta, 1);
 	setNeighborColor(Qt::blue);
 	setGlobalLoopClosureColor(Qt::red);
 	setLocalLoopClosureColor(Qt::yellow);
 	setUserLoopClosureColor(Qt::red);
 	setVirtualLoopClosureColor(Qt::magenta);
 	setNeighborMergedColor(QColor(255,170,0));
+	setRejectedLoopClosureColor(Qt::black);
 	setLandmarkColor(Qt::darkGreen);
+	setLocalPathColor(Qt::cyan);
+	setGlobalPathColor(Qt::darkMagenta);
+	setGTColor(Qt::gray);
+	setGPSColor(Qt::darkCyan);
+	setIntraSessionLoopColor(Qt::red);
+	setInterSessionLoopColor(Qt::green);
+	setIntraInterSessionColorsEnabled(false);
 	setGridMapVisible(true);
 	setGraphVisible(true);
 	setGlobalPathVisible(true);
@@ -1789,6 +1892,66 @@ void GraphViewer::wheelEvent ( QWheelEvent * event )
 	}
 }
 
+void GraphViewer::mouseMoveEvent(QMouseEvent * event)
+{
+	QPointF scenePoint = mapToScene(event->pos());
+	if(_mouseTracking && _viewPlane==XY && this->sceneRect().contains(scenePoint))
+	{
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+		QToolTip::showText(event->globalPosition().toPoint(), QString("%1m %2m").arg(-scenePoint.y()/100.0).arg(-scenePoint.x()/100.0));
+#else
+		QToolTip::showText(event->globalPos(), QString("%1m %2m").arg(-scenePoint.y()/100.0).arg(-scenePoint.x()/100.0));
+#endif
+	}
+	else
+	{
+		QToolTip::hideText();
+	}
+	if (event->modifiers() & (Qt::ShiftModifier | Qt::ControlModifier) && event->buttons() & Qt::LeftButton) {
+		// same modifiers than 3D view, change zoom
+		if(_previousMousePos.y()!=0) {
+			if(event->pos().y() - _previousMousePos.y() > 0)
+			{
+				this->scale(0.98, 0.98);
+			}
+			else
+			{
+				this->scale(1.02, 1.02);
+			}
+		}
+		_previousMousePos = event->pos();
+		return;
+	}
+	_previousMousePos.setY(0);
+	QGraphicsView::mouseMoveEvent(event);
+}
+
+void GraphViewer::mousePressEvent(QMouseEvent * event)
+{
+	QGraphicsItem *item = this->scene()->itemAt(mapToScene(event->pos()), QTransform());
+	if(item)
+	{
+		NodeItem *nodeItem = qgraphicsitem_cast<NodeItem*>(item);
+		LinkItem *linkItem = qgraphicsitem_cast<LinkItem*>(item);
+		if(nodeItem && nodeItem->parentItem() == _graphRoot && nodeItem->id() != 0)
+		{
+			Q_EMIT nodeSelected(nodeItem->id());
+		}
+		else if(linkItem && linkItem->parentItem() == _graphRoot && linkItem->from() != 0 && linkItem->to() != 0)
+		{
+			Q_EMIT linkSelected(linkItem->from(), linkItem->to());
+		}
+		else
+		{
+			QGraphicsView::mousePressEvent(event);
+		}
+	}
+	else
+	{
+		QGraphicsView::mousePressEvent(event);
+	}
+}
+
 QIcon createIcon(const QColor & color)
 {
 	QPixmap pixmap(50, 50);
@@ -1807,6 +1970,12 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 	QAction * aChangeNodeColor = menu.addAction(createIcon(_nodeColor), tr("Set node color..."));
 	QAction * aChangeNodeOdomCacheColor = menu.addAction(createIcon(_nodeOdomCacheColor), tr("Set node odom cache color..."));
 	QAction * aChangeCurrentGoalColor = menu.addAction(createIcon(_currentGoalColor), tr("Set current goal color..."));
+	QMenu * menuChangeHighlightingColors = menu.addMenu(tr("Set node highlighting colors..."));
+	QVector<QAction*> aChangeHighlightingColors(_highlightedNodes.size());
+	for(int i=0; i<_highlightedNodes.size(); ++i) {
+		aChangeHighlightingColors[i] = menuChangeHighlightingColors->addAction(createIcon(_highlightedNodes[i].first), tr("Color %1...").arg(i+1));
+		aChangeHighlightingColors[i]->setIconVisibleInMenu(true);
+	}
 	aChangeNodeColor->setIconVisibleInMenu(true);
 	aChangeNodeOdomCacheColor->setIconVisibleInMenu(true);
 	aChangeCurrentGoalColor->setIconVisibleInMenu(true);
@@ -1878,6 +2047,7 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 	QAction * aShowHideGPSGraph;
 	QAction * aShowHideOdomCacheOverlay;
 	QAction * aOrientationENU;
+	QAction * aMouseTracking;
 	QAction * aViewPlaneXY;
 	QAction * aViewPlaneXZ;
 	QAction * aViewPlaneYZ;
@@ -1976,6 +2146,10 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 	aOrientationENU = menu.addAction(tr("ENU Orientation"));
 	aOrientationENU->setCheckable(true);
 	aOrientationENU->setChecked(_orientationENU);
+	aMouseTracking = menu.addAction(tr("Show mouse cursor position (m)"));
+	aMouseTracking->setCheckable(true);
+	aMouseTracking->setChecked(_mouseTracking);
+	aMouseTracking->setEnabled(_viewPlane == XY);
 	aShowHideGraph->setEnabled(_nodeItems.size() && _viewPlane == XY);
 	aShowHideGraphNodes->setEnabled(_nodeItems.size() && _graphRoot->isVisible());
 	aShowHideGlobalPath->setEnabled(_globalPathLinkItems.size());
@@ -1999,6 +2173,13 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 	QAction * aRestoreDefaults = menu.addAction(tr("Restore defaults"));
 
 	QAction * r = menu.exec(event->globalPos());
+	int aChangeHighlightingColorIndex = 0;
+	for(int i=1; i<aChangeHighlightingColors.size(); ++i)
+	{
+		if(r == aChangeHighlightingColors[i]) {
+			aChangeHighlightingColorIndex = i;
+		}
+	}
 	if(r == aScreenShot)
 	{
 		if(_root)
@@ -2166,6 +2347,7 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 	else if(r == aChangeNodeColor ||
 			r == aChangeNodeOdomCacheColor ||
 			r == aChangeCurrentGoalColor ||
+			r == aChangeHighlightingColors[aChangeHighlightingColorIndex] ||
 			r == aChangeNeighborColor ||
 			r == aChangeGlobalLoopColor ||
 			r == aChangeLocalLoopColor ||
@@ -2193,6 +2375,10 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 		{
 			color = _currentGoalColor;
 		}
+		else if(r == aChangeHighlightingColors[aChangeHighlightingColorIndex])
+		{
+			color = _highlightedNodes[aChangeHighlightingColorIndex].first;
+		}
 		else if(r == aChangeGlobalLoopColor)
 		{
 			color = _loopClosureColor;
@@ -2265,6 +2451,10 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 			{
 				this->setCurrentGoalColor(color);
 			}
+			else if(r == aChangeHighlightingColors[aChangeHighlightingColorIndex])
+			{
+				this->setHighlightColor(color, aChangeHighlightingColorIndex);
+			}
 			else if(r == aChangeGlobalLoopColor)
 			{
 				this->setGlobalLoopClosureColor(color);
@@ -2405,6 +2595,10 @@ void GraphViewer::contextMenuEvent(QContextMenuEvent * event)
 	{
 		this->setOrientationENU(!this->isOrientationENU());
 	}
+	else if(r == aMouseTracking)
+	{
+		_mouseTracking = aMouseTracking->isChecked();
+	}
 	else if(r == aViewPlaneXY)
 	{
 		this->setViewPlane(XY);
